<#
.SYNOPSIS
    Creates a package containing Python for Dataiku Science Studio.
.DESCRIPTION
    Builds a package containing Python for Dataiku Science Studio by building python from the sources.
    Requires git to be installed.
.PARAMETER Version
    (Optional) The python version to build.
.PARAMETER OutputDir
    (Optional) The output directory, by default './python-win'.
.EXAMPLE
    C:\PS> ./build-embbeded-python.ps1 -Version 3.7.9
    C:\PS> ./build-embbeded-python.ps1 -Version 3.7.9 -OutputDir "C:/dev"
#>
param(
    [Parameter(Mandatory=$false)] [string] $Version = "3.7.13",
    [Parameter(Mandatory=$false)] [string] $OutputDir = "./python-win"
)

try
{
    git | Out-Null
}
catch [System.Management.Automation.CommandNotFoundException]
{
    throw "Git not installed or not in the PATH. This script requires Git to be installed and in the PATH. Check https://git-scm.com/download/win."
}

$PythonSourcesRepoUrl = "https://github.com/python/cpython"

$branches = git ls-remote "https://github.com/python/cpython.git"
$tags = [System.Linq.Enumerable]::Where($branches, [Func[object,bool]] { param($branch) $branch.Contains("refs/tags/") })
$tags = [System.Linq.Enumerable]::Select($tags, [Func[object, string]] { param($tag) $tag.Split("/")[2].Split("^")[0] } )
$tags = [System.Linq.Enumerable]::Where($tags, [Func[object,bool]] { param($branch) $branch.StartsWith("v") })
$tags = [System.Linq.Enumerable]::ToList($tags)

if (!$tags.Contains("v$Version"))
{
    Write-Error "The given version '$Version' does not exist. Specify one of the following versions (without the v): $tags"
    exit 1
}

$TmpDir = New-Item -ItemType Directory -Path (Join-Path ("./") ([System.Guid]::NewGuid()))
Write-Host "Created temporary directory: '$TmpDir'"

# Install build tools

$VSBuildToolsUrl = "https://aka.ms/vs/16/release/vs_buildtools.exe"
$VSBuildToolsFile = "$TmpDir/vs_BuildTools.exe"

Invoke-WebRequest $VSBuildToolsUrl -OutFile $VSBuildToolsFile

Write-Host "Downloaded $VSBuildToolsFile"

# We have to use the Start-Process command to wait for the installer process to finish
Start-Process -FilePath $VSBuildToolsFile -ArgumentList "--add", "Microsoft.VisualStudio.Workload.VCTools", "Microsoft.VisualStudio.Component.Windows10SDK.19041", "Microsoft.VisualStudio.Component.VC.140", "Microsoft.VisualStudio.Workload.MSBuildTools", "--passive", "--wait" -Wait

Write-Host "Installed required VS Build tools."

# Refresh the Path to include the newly installed MSBuild
$env:Path += ";C:\Program Files (x86)\Microsoft Visual Studio\2019\BuildTools\MSBuild\Current\Bin"

$RepoPath = "$TmpDir/cpython"

git clone -b "v$Version" $PythonSourcesRepoUrl $RepoPath

& "$RepoPath/Tools/nuget/build.bat" "-x64"

$NugetFile = "$RepoPath/PCBuild/amd64/python.$Version.nupkg"
$NugetZip = "$RepoPath/PCBuild/amd64/python.$Version.zip"

Move-Item -Path $NugetFile -Destination $NugetZip

Expand-Archive -Path $NugetZip -DestinationPath $TmpDir

mkdir -Force $OutputDir | Out-Null

Copy-Item "$PSCommandPath" -Destination "$TmpDir/tools/"

$PythonDir = "$TmpDir/python-$Version"

Move-Item -Path "$TmpDir/tools" -Destination $PythonDir

Compress-Archive -Force -Path $PythonDir -DestinationPath "$OutputDir/python-$Version-win.zip"

Write-Host "$OutputDir/python-$Version-win.zip"

Remove-Item -LiteralPath $TmpDir -Force -Recurse

Write-Host "Deleted '$TmpDir'"
